import json
from version_updater.parser import AbstractParser


class PackageParser(AbstractParser):
    """
        A parser for package.json files
    """

    def __init__(self, filename):
        super().__init__(filename)
        with open(filename) as f:
            js = json.load(f)
            self.js = js

    def get_version(self):
        return self.js['version']

    def set_version(self, new_version):
        self.js['version'] = new_version

    def set_dep_version(self, depname, new_version):
        self.js['dependencies'][depname] = new_version

    def return_content(self):
        return self.js

    def finish(self):
        with open(self.filename, 'w') as outf:
            outf.write(json.dumps(self.js, indent=2))



import argparse
from version_updater import PackageParser, YamlParser, SetupParser, RequirementsParser

"""
Supports:
- package.json
- .gitlab-ci.yml
- setup.py
- requirements.txt
"""


def get_parser(fname):
    if fname[-4:] == 'json':
        return PackageParser(fname)
    elif fname[-4:] == '.yml' or fname[-4:] == 'yaml':
        return YamlParser(fname)
    elif fname[-8:] == 'setup.py':
        return SetupParser(fname)
    elif len(fname) >= 16 and fname[-16:] == 'requirements.txt':
        return RequirementsParser(fname)
    else:
        return None


def core(filename, do_set, do_deps, bump):
    parser = get_parser(filename)
    if parser:
        if do_set:
            print(f'Setting {filename} to {do_set}')
            parser.set_version(do_set)
        elif bump:
            print(f'Bumping {filename} to {bump}')
            parser.bump(bump)

        if do_deps:
            dep_mappings = do_deps
            all_mappings = [{'dep': s[0], 'var': s[1]} for s in [
                s1.split('=') for s1 in dep_mappings.split(",")]]
            for kv in all_mappings:
                print(
                    f'Setting {filename} dependency {kv["dep"]} to {kv["var"]}')
                parser.set_dep_version(kv["dep"], kv["var"])
        parser.finish()
        print(f'Done!')
    else:
        print(f'Unable to handle file: {filename}')


def main():
    parser = argparse.ArgumentParser(
        description='Update semantic versions for artifacts')
    parser.add_argument('--set', help='Set the main artifact version')
    parser.add_argument('--set-dependencies', dest='set_deps',
                        help='Set a dependency artifact version')
    parser.add_argument('--bump', choices=['major', 'minor', 'patch'],
                        help='To bump the version instead of setting a specific value')
    parser.add_argument('file', metavar='FILE', type=str, nargs=1,
                        help='The file to use for the version info')
    args = parser.parse_args()
    filename = args.file[0]
    core(filename, args.set, args.set_deps, args.bump)


if __name__ == "__main__":
    main()



import semver


class AbstractParser:

    def __init__(self, filename):
        self.filename = filename

    def get_version(self):
        return ''

    def set_version(self, new_version):
        pass

    def set_dep_version(self, depname, new_version):
        pass

    def return_content(self):
        return None

    def finish(self):
        pass

    def bump(self, which):
        ver = self.get_version()
        semantic_ver = semver.VersionInfo.parse(ver)
        newver = semantic_ver
        if which.lower() == 'major':
            newver = semantic_ver.bump_major()
        elif which.lower() == 'minor':
            newver = semantic_ver.bump_minor()
        elif which.lower() == 'patch':
            newver = semantic_ver.bump_patch()
        final_ver = f'{newver.major}.{newver.minor}.{newver.patch}'
        print(f'Setting final version to {final_ver}')
        self.set_version(final_ver)



from version_updater.parser import AbstractParser


class RequirementsParser(AbstractParser):
    """
        A parser for requirements.txt files
    """

    def __init__(self, filename):
        super().__init__(filename)
        with open(filename) as f:
            self.contents = f.readlines()

    def set_dep_version(self, depname, new_version):
        ans = []
        for oneline in self.contents:
            splits = oneline.split('==')
            res = oneline
            if splits[0] == depname:
                res = f'{depname}=={new_version}\n'
            ans.append(res)
        self.contents = ans

    def bump(self, which):
        pass

    def return_content(self):
        return self.contents

    def finish(self):
        with open(self.filename, 'w') as outf:
            outf.write(''.join(self.contents))



import unittest
import shutil
from main import core

managed_file_fmts = ['test/test-data/{which}/requirements.txt',
                     'test/test-data/{which}/.gitlab-ci.yml',
                     'test/test-data/{which}/package1.json']


class TestParser(unittest.TestCase):

    def tearDown(self):
        for f in managed_file_fmts:
            shutil.copyfile(f.format(which='backup'), f.format(which='input'))

    # update-version --set-dependencies gtfs-realtime-bindings=4.5.6,jupyter=2.3.4 foo-requirements.txt --bump major
    def test_requirements_2_deps(self):
        deps = 'gtfs-realtime-bindings=4.5.6,jupyter=2.3.4'
        self.__test_core('requirements.txt', None, deps, 'major')

    # update-version --set-dependencies react=1.2.3,react-dom=2.3.4 package1.json --bump major
    def test_package_2_deps_bump_major(self):
        deps = 'react=1.2.3,react-dom=2.3.4'
        self.__test_core('package1.json', None, deps, 'major')

    # update-version --bump patch .gitlab-ci.yml
    def test_gl_yaml_bump_patch(self):
        self.__test_core('.gitlab-ci.yml', None, None, 'patch')

    # update-version --set-dependencies semver=4.5.6 setup.py --bump minor
    def test_setup_1_dep_bump_minor(self):
        self.__test_core('setup.py', None, 'semver=4.5.6', 'minor')

    def __test_core(self, which_file, do_set, deps, do_bump):
        input = f'test/test-data/input/{which_file}'
        expect = f'test/test-data/output/{which_file}'
        core(input, do_set, deps, do_bump)
        with open(input) as infile:
            test_contents = infile.read()
            with open(expect) as expect_file:
                expect_contents = expect_file.read()
                self.assertEqual(test_contents, expect_contents,
                                 f'Expected contents to be equal but were not.  Check {input} and compare to {expect}')


if __name__ == '__main__':
    unittest.main()



import json
from ruamel.yaml import YAML
from version_updater.parser import AbstractParser

yaml = YAML()
yaml.indent(offset=2)
yaml.allow_duplicate_keys = True


class YamlParser(AbstractParser):
    """
        A parser for gitlab-ci.yaml files
    """

    def __init__(self, filename):
        super().__init__(filename)
        with open(filename) as f:
            js = yaml.load(f)
            self.js = js

    def get_version(self):
        vars = self.js['variables']
        self.vvar = 'VERSION'
        for k in vars.keys():
            if k.lower() == 'version':
                self.vvar = k
        return vars[self.vvar]

    def set_version(self, new_version):
        self.get_version()
        self.js['variables'][self.vvar] = new_version

    def return_content(self):
        return self.js

    def finish(self):
        with open(self.filename, 'w') as outf:
            yaml.dump(self.js, outf)



import json
from ruamel.yaml import YAML
from version_updater.parser import AbstractParser
import re

yaml = YAML()
reg_format = '({tok}\\s*==?\\s*)("|\\\')?([0-9]+[.][0-9]+[.][0-9]+)("|\\\')?'


class SetupParser(AbstractParser):
    """
        A parser for setup.py files
    """

    def __init__(self, filename):
        super().__init__(filename)
        with open(filename) as f:
            self.contents = f.read()

    def get_version(self):
        regc = re.compile(reg_format.format(tok='version'))
        mo = regc.search(self.contents)
        return mo.group(3)

    def set_version(self, new_version):
        self.__subs_core('version', new_version)

    def set_dep_version(self, depname, new_version):
        self.__subs_core(depname, new_version)

    def __subs_core(self, depname, new_version):
        regc = re.compile(reg_format.format(tok=depname))
        mo = regc.search(self.contents)
        fstr = f'{mo.group(1)}{mo.group(2)}{new_version}{mo.group(4)}' if mo.group(
            2) and mo.group(4) else f'{mo.group(1)}{new_version}{mo.group(4)}'
        subst = regc.sub(
            fstr, self.contents)
        self.contents = subst

    def return_content(self):
        return self.contents

    def finish(self):
        with open(self.filename, 'w') as outf:
            outf.write(self.contents)



from setuptools import setup, find_packages

entrypoints = {}

console_scripts = entrypoints['console_scripts'] = [
    'update-version = version_updater.main:main',
]

try:
    long_description = open('README.md').read()
except IOError:
    long_description = ''

setup(
    name='version_updater',
    version='0.0.3',
    description="Updates semantic versions, either of the artifact itself, or its dependencies",
    author="Jeremy Vickery",
    author_email="jeremyvickery@gmail.com",
    platforms=['any'],
    license='MIT',
    packages=find_packages(),
    zip_safe=False,
    install_requires=[
        'semver==2.13.0',
        'jsonpath-ng==1.5.2',
        'ruamel.yaml==0.16.12'
    ],
    include_package_data=True,
    entry_points=entrypoints,
    long_description=long_description
)
